<?php
/**
 * @package    JDiDEAL
 *
 * @author     Roland Dalmulder <contact@rolandd.com>
 * @copyright  Copyright (C) 2009 - 2023 RolandD Cyber Produksi. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link       https://rolandd.com
 */

use Jdideal\Gateway;
use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\Registry\Registry;

defined('_JEXEC') or die;

/**
 * JoomDonation payment plugin.
 *
 * @package  JDiDEAL
 * @since    3.0
 */
class os_jdideal extends os_payment
{
    /**
     * Get the profile alias.
     *
     * @var    string
     * @since  4.0
     */
    private $profileAlias = '';

    /**
     * Constructor, initialise some parameters.
     *
     * @param Registry $params An object with parameters settings
     *
     * @since   3.0
     */
    public function __construct($params)
    {
        parent::setName('os_jdideal');
        parent::os_payment();
        parent::setCreditCard(false);
        parent::setCardType(false);
        parent::setCardCvv(false);
        parent::setCardHolderName(false);

        $this->profileAlias = $params->get('profileAlias');
    }

    /**
     * Process the payment request.
     *
     * @param object $row The payment details row.
     * @param array $data An array with payment information.
     *
     * @return string The HTML payment form.
     *
     * @throws  Exception
     * @since   3.0
     *
     */
    public function processPayment($row, $data)
    {
        $jinput = JFactory::getApplication()->input;
        $Itemid = $jinput->get('Itemid', 1, 'int');

        // Set some needed data
        $data = array(
            'amount' => number_format($data['gateway_amount'], 2, '.', ''),
            'order_number' => substr($data['first_name'] . ' ' . $data['last_name'], 0, 32),
            'order_id' => $row->id,
            'origin' => 'joomdonation',
            'return_url' => JUri::base(
                ) . 'index.php?option=com_jdonation&view=complete&id=' . $row->id . '&Itemid=' . $Itemid,
            'notify_url' => JUri::base(
                ) . 'index.php?option=com_jdonation&task=payment_confirm&payment_method=os_jdideal',
            'cancel_url' => JUri::base(
                ) . 'index.php?option=com_jdonation&view=cancel&id=' . $row->id . '&Itemid=' . $Itemid,
            'email' => '',
            'payment_method' => '',
            'profileAlias' => $this->profileAlias,
            'custom_html' => '',
            'silent' => false,
        );

        // Build the form
        ob_start();

        // Include the payment form
        $layout = new JLayoutFile('forms.form', null, array('component' => 'com_jdidealgateway'));
        echo $layout->render(array('data' => $data));
        $html = ob_get_contents();

        ob_end_clean();

        echo $html;
    }

    /**
     * Verify the payment.
     *
     * @return  void.
     *
     * @throws  Exception
     * @throws  RuntimeException
     * @throws  InvalidArgumentException
     * @throws  UnexpectedValueException
     * @since   2.14
     *
     */
    public function verifyPayment()
    {
        JLoader::registerNamespace('Jdideal', JPATH_LIBRARIES);

        if (class_exists(Gateway::class) === false) {
            JLoader::registerNamespace('Jdideal', JPATH_LIBRARIES . '/Jdideal');
        }

        $jinput = Factory::getApplication()->input;
        $jdideal = new Gateway;

        $trans = $jinput->get('transactionId');
        $column = 'trans';

        if (empty($trans)) {
            $trans = $jinput->get('pid');
            $column = 'pid';
        }

        $details = $jdideal->getDetails($trans, $column, false, 'joomdonation');

        // Set the status
        switch ($jdideal->getStatusCode($details->result)) {
            case 'C':
                $published = 1;
                break;
            default:
                $published = 0;
                break;
        }

        // Update the registration
        $app = Factory::getApplication('site');
        $config = JoomDonationHelper::getConfig();
        $id = $details->order_id;
        $transactionId = $trans;
        $amount = $details->amount;

        if ($amount < 0) {
            $jdideal->log('No amount. Stop processing.', $details->id);

            return false;
        }

        $row = JTable::getInstance('jdonation', 'Table');
        $row->load($id);

        if (!$row->id) {
            return false;
        }

        $row->transaction_id = $transactionId;
        $row->payment_date = date('Y-m-d H:i:s');
        $row->published = $published;
        $row->store();

        // All is good, return
        if ($published === 1) {
            PluginHelper::importPlugin('jdonation');

            if (JVERSION < 4) {
                $dispatcher = JEventDispatcher::getInstance();
                $dispatcher->trigger('onAfterPaymentSuccess', [$row]);
            } else {
                Factory::getApplication()->triggerEvent('onAfterPaymentSuccess', [$row]);
            }

            JoomDonationHelper::sendEmails($row, $config);
            $app->redirect($details->return_url);
        } else {
            $app->redirect($details->cancel_url);
        }
    }
}
